﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using System.IO;

/////////stl-central.com/////////
//(c) 2009 gdata & nEw-GEN Mods//
///////////ngmods.com////////////

/*First of all here is a short introduction and I will tell you about a few things you should take care of:
 
////Attributes////
 * If a attribute does not exist, it will be appended to the end of the document!
 * Make sure you specify the type of the attribute!
 * If your attribute does not exist, or there is a spelling mistake an empty string or array will be returned. Check that!
 
////Return Values////
 * Most methods return a string or an array of strings(eg. GetAttribute() and GetAttributeArray()) even if your attribute
 * should be a float or int value. This has been done to make the whole thing less chaotic, but it should be possible to convert
 * these values, using the class System.Convert.
 * example:
       float PulseEnergy = Convert.ToSingle(odf.GetAttribute("energyPerPulsePhaser", OdfAttributeType.Float));
 * If you are not sure whether this works, because of inconsistent odf scripting by modders ;), you should catch exceptions by try/catch.
 * Also, when working directly with the Parser, I recommend changing the decimal seperator (There are methods for that in the OdfParser class).
 * See also below:

////Float Values and Localization////
 * If your local decimal seperator is a comma, you may also use that, for example when you convert a float to a string.
 * You will also get your local decimal seperator if you read floate values! That makes the conversion  from string to float easier.
 * When you work directly with the Parser, you should change the decimal seperator to International when feeding the parser
 * with converted float values, and change the decimal seperator to Local, when using parsed values in float variables.
 * If you're from the US (or anywhere else, where a dot is used as decimal seprator), this does not apply for you. (lucky you ;))
 
////Indices////
 * If you access a line via its index, you should consider that these indices are zero based and:
 * If your index is above the actual amount of lines, nothing will happen.
 * Remember that these indices are flexible! If you Append, Insert or Remove Lines, Comments or Attributes they change!
 * Let's say you want to remove the first three lines. You can't do it like this:
       for (int i = 0; i < 3; i++)
            RemoveLine(i);
 * because the second line will be the first line, after the first has been removed. Therefore I recommend doing it like this:
       for (int i = 0; i < numberoflines; i++)
            RemoveLine(startindex);
 * numberoflines would be 3 and start index 0, when trying to remove the first three lines.
 * Alternativly you could arrange the indices in descending order (for example if your indices are spreaded through the whole file)
 * But since it's possible to remove comments, includes and attributes manually, the RemoveLine() method should be obsolete.


////Comments////
 * You can add comments to lines, attributes and also insert them by using indices
 * If a string in a method is specified as 'comment' you don't need to add the double slash
 * I recommend not to mess a lot with them, especially with the UnComment() method, since this might easily cause syntax error (not my fault though ;))

////OdfParser & Document - Differences////
 * There are two classes in the nEw-GEN.Odf namespace: OdfParser and OdfDocument
 * The Parser is static and can only process single lines, the OdfDocument is non-static and can process whole files.
 * Even though the OdfDocument has been optimiized for speed, I recommend using the parser when working with a lot of files (50+)
 * Using the Parser can be more difficult though.
 * Also, don't forget to remove the comments first, when you try getting values with the Parser. The Document does that automatically.

////Armada/Legacy support////
 * This supports Legacy and Armada, as both engines work with odf files.
 * This example refers to Legacy, but it can be easily adapted for Armada (just change the odf and the respective attributes in the code)

////Problems, Bugs, etc.////
 * If you find a bug please e-mail me (gdata@ngmods.com) or post here: http://stl-central.com
 * If you have problems using this, I recommend you post on our forums: http://stl-central.com.
 * Please don't email me if it's not a bug

////Usage Policy////
 * http://creativecommons.org/licenses/by-nc-sa/3.0/
 * You may use my supplied source code, but if you do, you need to publish it as well
 * You may not modify the library itself.
 * Using OOP-techniques, such as inheritance, to expand the library's functionality is allowed
 * According to the Creative Commons you have to credit me

////Disclaimer////
 * This material is not made or supported by Bethesda or MAD DOC Software; it is freeware and may not be used for commercial purposes!
 * By installing this addon, you (the user) agree to and assume all risks of said addon. As a result, we/I take no responsibility for
 * any damages that may occur to your game, mod or hardware.
 * Star Trek and related marks are trademarks of Paramount Pictures/CBS.All rights reserved. All other trademarks and trade names are
 * the properties of their respective owners.
*/

namespace nEwGEN.Odf
{
    class Program
    {
        static void Main(string[] args)
        {
            File.Copy("F_Discovery.backup", "F_Discovery.odf", true); //Let's restore the old file, to undo changes from the last edit.
            Console.WriteLine("Welcome to gdata's OdfLibary Example Application...");
            OdfDocument odf = new OdfDocument("F_Discovery.odf"); //This loads the odf from the Startup Directory.

            //First we want some statistics...
            Console.WriteLine("...example file: F_Discovery.odf");
            Console.WriteLine("Press ENTER to continue...");
            Console.WriteLine();
            Console.WriteLine("Number of Includes: " + odf.CountIncludes());
            Console.WriteLine("Number of Attributes: " + odf.CountAttributes());
            Console.WriteLine("Number of Lines: " + odf.CountLines());
            Console.ReadLine();

            //Now we will take a look at the reading functionalities of this tool...we will edit later.
            string[] atts = new string[] { "unitName", "race", "weapon1", "weapon2", "weapon3", "weapon4", "weapon15" };
            foreach (string att in atts)
                if (odf.GetAttribute(att, OdfAttributeType.String) != string.Empty)
                    Console.WriteLine(att + ": " + odf.GetAttribute(att, OdfAttributeType.String));
                else
                    Console.WriteLine(att + ": not found!");

            Console.ReadLine();

            //Now it's time for some float values
            atts = new string[] { "energyPerPulsePhaser", "weaponsEnergyReplenishRate", "weaponPoolDiminishRate" };
            foreach (string attr in atts)
                if (odf.GetAttribute(attr, OdfAttributeType.Float) != string.Empty)
                    Console.WriteLine(attr + ": " + odf.GetAttribute(attr, OdfAttributeType.Float));
                else
                    Console.WriteLine(attr + ": not found!");

            Console.ReadLine();

            //This is a special...a MultilineStringArray....took ages to implement this.
            string attri = "possibleCraftNames";
            Console.WriteLine(attri + ": ");
            if(odf.AttributeExists(attri))
                foreach (string craftname in odf.GetAttributeArray(attri, OdfAttributeArrayType.StringMultiLine))
                    Console.WriteLine(craftname);
            Console.ReadLine();

            //Includes
            Console.WriteLine("Includes:");
            string[] includes = odf.GetIncludes();
            foreach (string include in includes)
                    Console.WriteLine(include);
            Console.ReadLine();

            //Okay, now let's get a list of all attributes in there...
            Console.WriteLine("All Attributes from this file:");
            string[] attlist = odf.GetAttributeList();
            foreach (string attr in attlist)
                    Console.WriteLine(attr);
            Console.WriteLine();
            Console.WriteLine("Let's make some changes to the odf now. Press enter to view odf");
            Console.ReadLine();

            //This will change the craftnames and add a shieldColor attribute (since this does not exist yet)
            odf.SetAttributeArray("possibleCraftNames", new string[] { "GDATA", "WHO", "IS", "THE", "GREATEST", "MODDER", "EVER", "MADE", "THIS", "FREAKING", "LIBRARY" }, OdfAttributeArrayType.StringMultiLine, "gdata made some weird changes here");
            odf.SetAttributeArray("shieldColor", new string[]{"0.5", "0.4", "0.8"}, OdfAttributeArrayType.Float); 

            //Now let's remove the sensors attribute...but before whe do we should mark that place! We could also comment it out, but we don't want to do that.
            //Alternatively we could simply replace the line using the IndexOfAttribute() and the ReplaceLine() method.
            odf.InsertComment(odf.IndexOfAttribute("HasSensors"), "This is where the hasSensors attribute was!");
            odf.RemoveAttribute("hasSensors", false);

            //The attribute MassScale appears three times...let's change that! (Note: Both engines, Armada and Legacy, use the first attribute, so this will keep the first one and remove the others)
            odf.MakeUniqueAttribute("MassScale", true);

            //Insert a random line at the center of the file. You should not use this, since there is an own method
            //for comments and attributes should be added via SetAttribute()
            odf.InsertLine(odf.CountLines()/2, "//This should be at the mid of the file!");
            //Same as above, just for comments...
            odf.InsertComment(odf.CountIncludes(), "Welcome to this file!!! Above are the Includes");

            //Pretty self-explainatory...
            odf.AppendComment("This file has been changed on: " + DateTime.Now.ToString());
            odf.AppendCommentToAttribute("commandPointCost", "This is our price!");
            odf.AppendCommentToLine(15, "Let's add something here!");
            odf.AppendInclude("this_include_does_not_exist.odf");

            //This will enable the file again, since it was 0 before
            odf.SetAttribute("commandPointCost", "500", OdfAttributeType.Int);

            //Remember the crap I added as craftnames? Let's comment that out!
            odf.RemoveAttribute("possibleCraftNames", true);

            //Now let's take a look at our changes...
            for (int i = 0; i < odf.CountLines(); i++)
                Console.WriteLine(odf.GetLine(i));

            odf.Save("F_Discovery_changed.odf");
            //Uncomment the next line if you want to use the editor to view the file.
            //Process.Start("F_Discovery_changed.odf");
            Console.WriteLine("Press Enter to Exit");
            Console.ReadLine();
        }
    }
}
